{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Request activation link" context "account activation page" %} | {{ block.super }}{% endblock %}


{% block content %}
<div class="page page-request-activation">
  <div class="container page-header-container">
    <div class="page-header page-header-request-activation">
      <div class="page-header-bg-image">
        <div class="page-header-bg-overlay">
          <div class="page-header-image"></div>
          <div class="page-header-banner page-header-banner-request-activation">
            <div class="page-header-banner-bg-image">
              <div class="page-header-banner-bg-overlay">
                <h1>{% trans "Request activation link" context "account activation page" %}</h1>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
  <div class="container page-container">
    <div class="row">
      <div class="col-md-8">
        <div class="side-form-copy">

          <p class="lead">
            {% trans "The site administrator may impose a requirement on newly registered accounts to be activated before users will be able to sign in." context "account activation page" %}
          </p>
          <p>
            {% trans "Depending on the time of registration, you will be able to activate your account by clicking a special activation link. This link will be valid only for your browser, for seven days or until your account is activated." context "account activation page" %}
          </p>
          <p>
            {% blocktrans trimmed context "account activation page" %}
              To receive this link, enter your account's e-mail address and press the "Send link" button.
            {% endblocktrans %}
          </p>

        </div>
      </div>
      <div id="request-activation-link-mount" class="col-md-4">

        <noscript>
          <div class="well well-form well-noscript">
            <div class="noscript-message">

              <div class="message-icon">
                <span class="material-icon">error_outline</span>
              </div>

              <div class="message-body">
                <p>{% trans "To request a new activation link please enable JavaScript." context "account activation page" %}</p>
              </div>

            </div>
          </div>
        </noscript>

      </div>
    </div>

  </div>
</div>
{% endblock content %}
