{% extends "misago/admin/generic/form.html" %}
{% load i18n misago_admin_form %}


{% block title %}
{% if target.pk %}
  {{ target }}
{% else %}
  {% trans "New category role" context "admin category role form" %}
{% endif %} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block page-header %}
{{ block.super }}
{% if target.pk %}
  <small>
    {{ target }}
  </small>
{% endif %}
{% endblock page-header %}


{% block form-header %}
{% if target.pk %}
  {% trans "Edit category role" context "admin category role form" %}
{% else %}
  {% trans "New category role" context "admin category role form" %}
{% endif %}
{% endblock %}


{% block form-body %}
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Basic settings" context "admin category role form" %}</legend>
    {% form_row form.name %}
  </fieldset>
</div>
{% for form in perms_forms %}
  {% include "misago/admin/permissions_table.html" %}
{% endfor %}
{% endblock form-body %}
