# Generated by Django 1.11.16 on 2018-11-25 15:31
from django.db import migrations

from ...cache.operations import StartCacheVersioning


class Migration(migrations.Migration):
    dependencies = [
        ("misago_users", "0015_user_agreements"),
        ("misago_cache", "0001_initial"),
    ]

    operations = [StartCacheVersioning("bans")]
