{% load i18n misago_avatars %}
<div class="post-heading">
  {% if not post.is_read %}
    <span class="label label-unread hidden-xs">
      {% trans "New post" context "post unread label" %}
    </span>
    <span class="label label-unread visible-xs-inline-block">
      {% trans "New" context "post unread label" %}
    </span>
  {% endif %}

  <a href="{{ post.get_absolute_url }}" class="btn btn-link posted-on hidden-xs">
    <time itemprop="datePublished" datetime="{{ post.posted_on|date:'c' }}">
      {{ post.posted_on|date:'DATETIME_FORMAT' }}
    </time>
  </a>
  <a href="{{ post.get_absolute_url }}" class="btn btn-link posted-on visible-xs-inline-block">
    {{ post.posted_on|date:'DATETIME_FORMAT' }}
  </a>

  <div class="pull-right">
    <a href="{{ post.get_absolute_url }}" title="{% trans 'Post link' context 'post permalink' %}" class="btn btn-default btn-icon">
      <span class="material-icon">
        link
      </span>
    </a>
  </div>
</div>