# Generated by Django 3.2.15 on 2023-04-06 18:04

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("misago_users", "0023_remove_user_sso_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="notify_new_private_threads_by_followed",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "Don't notify"),
                    (1, "Notify on site only"),
                    (2, "Notify on site and with e-mail"),
                ],
                default=2,
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="notify_new_private_threads_by_other_users",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "Don't notify"),
                    (1, "Notify on site only"),
                    (2, "Notify on site and with e-mail"),
                ],
                default=2,
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="unread_notifications",
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name="user",
            name="watch_new_private_threads_by_followed",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "Don't notify"),
                    (1, "Notify on site only"),
                    (2, "Notify on site and with e-mail"),
                ],
                default=2,
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="watch_new_private_threads_by_other_users",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "Don't notify"),
                    (1, "Notify on site only"),
                    (2, "Notify on site and with e-mail"),
                ],
                default=2,
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="watch_replied_threads",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "Don't notify"),
                    (1, "Notify on site only"),
                    (2, "Notify on site and with e-mail"),
                ],
                default=2,
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="watch_started_threads",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "Don't notify"),
                    (1, "Notify on site only"),
                    (2, "Notify on site and with e-mail"),
                ],
                default=2,
            ),
        ),
    ]
