# Generated by Django 4.2.7 on 2023-12-05 19:22

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "misago_threads",
            "0013_rename_pollvote_poll_voter_name_misago_thre_poll_id_c3e8fe_idx_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="attachment",
            name="plugin_data",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="attachmenttype",
            name="plugin_data",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="poll",
            name="plugin_data",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="post",
            name="plugin_data",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="thread",
            name="plugin_data",
            field=models.JSONField(default=dict),
        ),
        migrations.AddIndex(
            model_name="attachment",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["plugin_data"], name="misago_thre_plugin__46d25a_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="attachmenttype",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["plugin_data"], name="misago_thre_plugin__236db5_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="poll",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["plugin_data"], name="misago_thre_plugin__5e1be7_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="post",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["plugin_data"], name="misago_thre_plugin__91da06_gin"
            ),
        ),
        migrations.AddIndex(
            model_name="thread",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["plugin_data"], name="misago_thre_plugin__e6e836_gin"
            ),
        ),
    ]
