{% load i18n %}
<div class="toolbar">
  <div class="toolbar-section hidden-xs">
    <div class="toolbar-item">
      <div class="dropdown">
        <button class="btn btn-default btn-outline btn-icon" title="{%trans 'Shortcuts' context 'thread page options' %}" type="button" disabled>
          <span class="material-icon">bookmark</span>
        </button>
      </div>
    </div>
    {% if paginator.pages > 1 %}
      <div class="toolbar-item">
        {% include "misago/thread/paginator.html"%}
      </div>
    {% endif %}
  </div>
  <div class="toolbar-spacer hidden-xs"></div>
  {% if user.is_authenticated %}
    {% if thread.acl.can_start_poll %}
      <div class="toolbar-section hidden-xs">
        <div class="toolbar-item">
          <button class="btn btn-default btn-outline btn-block" type="button" disabled>
            <span class="material-icon">poll</span>
            {% trans "Add poll" context "thread page options" %}
          </button>
        </div>
      </div>
    {% endif %}
    <div class="toolbar-section">
      <div class="toolbar-item hidden-sm hidden-md hidden-lg toolbar-item-shrink">
        <div class="dropdown">
          <button class="btn btn-default btn-outline btn-icon" title="{% trans 'Shortcuts' context 'thread page options' %}" type="button" disabled>
            <span class="material-icon">bookmark</span>
          </button>
        </div>
      </div>
      {% if thread.acl.can_reply %}
        <div class="toolbar-item">
          <button class="btn btn-primary btn-outline btn-block" type="button" disabled>
            <span class="material-icon">chat</span>
            {% trans "Reply" context "thread page options" %}
          </button>
        </div>
      {% endif %}
      {% if thread.acl.can_start_poll %}
        <div class="toolbar-item hidden-sm hidden-md hidden-lg toolbar-item-shrink">
          <button class="btn btn-default btn-outline btn-icon" title="{% trans 'Add poll' context 'thread page options' %}" type="button" disabled>
            <span class="material-icon">poll</span>
          </button>
        </div>
      {% endif %}
    </div>
  {% endif %}
  {% if paginator.pages > 1 %}
    <div class="toolbar-section hidden-sm hidden-md hidden-lg">
      <div class="toolbar-item">
        {% include "misago/thread/paginator.html"%}
      </div>
    </div>
  {% endif %}
</div>