{% load i18n %}
<div class="modal fade" id="uploadMedia" tabindex="-1" role="dialog">
  <div class="modal-dialog" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h5 class="modal-title">{% trans "Upload media" context "admin theme media upload form" %}</h5>
        <button type="button" class="close" data-dismiss="modal" aria-label="{% trans 'Close' context 'admin theme media upload form' %}">
          <span aria-hidden="true">&times;</span>
        </button>
      </div>
      <form action="{% url 'misago:admin:themes:upload-media' pk=theme.pk %}" method="post" enctype="multipart/form-data">
        {% csrf_token %}
        <div class="modal-body">
          <div class="form-group">
            <label for="uploadMediaInput">{% trans "Select media files to upload" context "admin theme media upload form" %}:</label>
            <input type="file" class="form-control" id="uploadMediaInput" name="assets" multiple>
            <small class="form-text text-muted">
              {% trans "Media files are primarily image and font files that can be linked to from the CSS, but files of any type can be uploaded using this option." context "admin theme media upload form" %}
            </small>
            <small class="form-text text-muted">
              {% trans 'Theme CSS that use the "url()" to point to media files will be updated automatically.' context "admin theme media upload form" %}
            </small>
          </div>
        </div>
        <div class="modal-footer">
          <button type="button" class="btn btn-light" data-dismiss="modal">{% trans "Cancel" context "admin theme media upload form" %}</button>
          <button class="btn btn-primary">{% trans "Upload" context "admin theme media upload form" %}</button>
        </div>
      </form>
    </div><!-- /.modal-content -->
  </div><!-- /.modal-dialog -->
</div><!-- /.modal -->