{% extends "misago/emails/base.html" %}
{% load i18n misago_absoluteurl misago_capture %}


{% block content %}
{% blocktrans trimmed with user=user context "account activated email" %}
{{ user }}, your account has been activated by site administrator.
{% endblocktrans %}
<br>
<br>
{% capture trimmed as login_link %}
<a href="{% absoluteurl LOGIN_URL %}?modal=login">{% trans "this form" context "account activated email from admin sign in cta link" %}</a>
{% endcapture %}
{% blocktrans trimmed with login_form=login_link|safe context "account activated email from admin sign in cta" %}
You can now sign in to it using {{ login_form }}.
{% endblocktrans %}
<br>
{% endblock content %}
