{% extends "misago/emails/base.html" %}
{% load i18n misago_absoluteurl %}


{% block content %}
{% blocktrans trimmed with user=user context "confirm email change email" %}
{{ user }}, you are receiving this message because you have changed your e-mail address.
{% endblocktrans %}
<br>
<br>
{% blocktrans trimmed context "confirm email change email" %}
To confirm this change, click the link below:
{% endblocktrans %}
<br>
<br>
<a href="{% absoluteurl 'misago:options-confirm-email-change' token=token %}">{% trans "Confirm change" context "confirm email change email cta" %}</a>
<br>
{% endblock content %}
