{% load i18n misago_absoluteurl %}
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html lang="{{ LANGUAGE_CODE }}">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
    <meta name="viewport" content="initial-scale=1.0">    <!-- So that mobile webkit will display zoomed in -->
    <meta name="format-detection" content="telephone=no"> <!-- disable auto telephone linking in iOS -->

    <style type="text/css">

        /* Resets: see reset.css for details */
        .ReadMsgBody { width: 100%; background-color: #eff0f1;}
        .ExternalClass {width: 100%; background-color: #eff0f1;}
        .ExternalClass, .ExternalClass p, .ExternalClass span, .ExternalClass font, .ExternalClass td, .ExternalClass div {line-height:100%;}
        body {-webkit-text-size-adjust:none; -ms-text-size-adjust:none;}
        body {margin:0; padding:0;}
        table {border-spacing:0;}
        table td {border-collapse:collapse;}
        .yshortcuts a {border-bottom: none !important;}


        /* Constrain email width for small screens */
        @media screen and (max-width: 600px) {
            table[class="container"] {
                width: 95% !important;
            }
        }

        /* Give content more room on mobile */
        @media screen and (max-width: 480px) {
            td[class="container-padding"] {
                padding-left: 12px !important;
                padding-right: 12px !important;
            }
        }

    </style>
</head>
<body style="margin:0; padding:10px 0;" bgcolor="#eff0f1" leftmargin="0" topmargin="0" marginwidth="0" marginheight="0">

<br>

<!-- 100% wrapper (grey background) -->
<table border="0" width="100%" height="100%" cellpadding="0" cellspacing="0" bgcolor="#eff0f1">
  <tr>
    <td align="center" valign="top" bgcolor="#eff0f1" style="background-color: #eff0f1;">

      <!-- 600px container (white background) -->
      <table border="0" width="600" cellpadding="0" cellspacing="0" class="container" bgcolor="#ffffff">
        <tr>
          <td class="container-padding" bgcolor="#ffffff" style="background-color: #ffffff; border: 1px solid #dde0e2; padding-left: 30px; padding-right: 30px; font-size: 13px; line-height: 20px; font-family: Helvetica, sans-serif; color: #333; padding-top: 4px;" align="left">
            <br>

            <!-- ### BEGIN CONTENT ### -->

              <table border="0" width="100%" height="100%" cellpadding="0" cellspacing="0">
                <tr>
                  <td valign="middle" style="font-size: 28px; line-height: 24px; color: #555555;">{{ settings.forum_name }}</td>
                  <td align="center" valign="middle" width="30"><img src="{% absoluteurl 'misago:user-avatar' pk=user.pk size=32 %}" width="32" height="32" style="border-radius: 3px;" alt=""></td>
                </tr>
              </table>
            <br>

            <div style="font-weight: bold; font-size: 18px; line-height: 24px; color: #333; border-top: 1px solid #ddd;"><br>{% block title %}{{ subject }}{% endblock %}</div>
            <br>

            {% block content %}{% endblock content %}

            <br>
            <div style="border-top: 1px solid #ddd; color: #666; font-size: 12px; line-height: 18px;">
              {% if settings.email_footer %}<br>{{ settings.email_footer }}{% endif %}
              <br><a href="{{ settings.forum_address }}" style="color: #888; text-decoration: underline;">
                {% blocktrans trimmed with forum_host=forum_host context "email footer" %}
                Sent from {{ forum_host }}
                {% endblocktrans %}
              </a>
            </div>

            <!-- ### END CONTENT ### -->
            <br>

          </td>
        </tr>
      </table>
      <!--/600px container -->

    </td>
  </tr>
</table>
<!--/100% wrapper-->
<br>
<br>
</body>
</html>
