# Generated by Django 3.2.15 on 2023-01-04 12:38

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("misago_users", "0022_deleteduser"),
        ("misago_oauth2", "0002_copy_sso_subjects"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="user",
            name="sso_id",
        ),
    ]
