{% load i18n %}
<nav class="toolbar thread-toolbar-third">
  <div class="toolbar-spacer"></div>
  <div class="toolbar-section">
    <div class="toolbar-item">
      <a href="#" class="btn btn-muted btn-block">
        <span class="material-icon">arrow_upward</span>
        {% trans "Go to top" context "go up" %}
      </a>
    </div>
  </div>
</nav>