{% load i18n misago_capture %}
<ul class="list-unstyled list-inline poll-details">
  <li class="poll-info-votes">
    {% blocktrans trimmed count votes=poll.votes context "thread poll" %}
      {{ votes }} vote.
    {% plural %}
      {{ votes }} votes.
    {% endblocktrans %}
  </li>
  {% if poll.length %}
    <li class="poll-info-ends-on">
      {% capture trimmed as ends_on %}
        <abbr title="{{ poll.ends_on }}">{{ poll.ends_on|date }}</abbr>
      {% endcapture %}
      {% blocktrans trimmed with ends_on=ends_on|safe context "thread poll" %}
        Voting ends {{ ends_on }}.
      {% endblocktrans %}
    </li>
  {% endif %}
  {% if poll.is_public %}
    <li class="poll-info-public">
      {% trans "Voting is public." context "thread poll" %}
    </li>
  {% endif %}
  <li class="poll-info-creation">
    {% capture trimmed as poster %}
      {% if poll.poster_id %}
        <a href="{% url 'misago:user' pk=poll.poster_id slug=poll.poster_slug %}" class="item-title">{{ poll.poster_name }}</a>
      {% else %}
        <span class="item-title">{{ poll.poster_name }}</span>
      {% endif %}
    {% endcapture %}
    {% capture trimmed as posted_on %}
      <abbr title="{{ poll.posted_on }}">{{ poll.posted_on|date }}</abbr>
    {% endcapture %}
    {% blocktrans trimmed with poster=poster|safe posted_on=posted_on|safe context "thread poll" %}
      Started by {{ poster }} on {{ posted_on }}.
    {% endblocktrans %}
  </li>
</ul>