{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Users" context "users page" %} | {{ block.super }}{% endblock %}


{% block content %}
<div class="page page-users-lists">
  <div class="container page-header-container">
    <div class="page-header page-header-users-lists">
      <div class="page-header-bg-image">
        <div class="page-header-bg-overlay">
          <div class="page-header-image"></div>
          <div class="page-header-banner page-header-banner-users-lists">
            <div class="page-header-banner-bg-image">
              <div class="page-header-banner-bg-overlay">
                <h1>{% trans "Users" context "users page" %}</h1>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
  {% block page-content %}{% endblock page-content %}
</div>
{% endblock content %}
