{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}
{% if user.is_authenticated and not user.has_usable_password %}
  {% trans "Set new password" context "set new password form page" %} | {{ block.super }}
{% else %}
  {% trans "Change forgotten password" context "forgotten password form page" %} | {{ block.super }}
{% endif %}
{% endblock %}


{% block content %}
<div class="page page-forgotten-password-form">
  <div class="container page-header-container">
    <div class="page-header page-header-forgotten-password-form">
      <div class="page-header-bg-image">
        <div class="page-header-bg-overlay">
          <div class="page-header-image"></div>
          <div class="page-header-banner page-header-banner-forgotten-password-form">
            <div class="page-header-banner-bg-image">
              <div class="page-header-banner-bg-overlay">
                {% if user.is_authenticated and not user.has_usable_password %}
                  <h1>{% trans "Set new password" context "set new password form page" %}</h1>
                {% else %}
                  <h1>{% trans "Change forgotten password" context "forgotten password form page" %}</h1>
                {% endif %}
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
  <div class="container page-container">

    <div class="row">
      <div id="reset-password-form-mount" class="col-md-4 col-md-offset-4">

        <noscript>
          <div class="well well-form well-noscript">
            <div class="noscript-message">

              <div class="message-icon">
                <span class="material-icon">error_outline</span>
              </div>

              <div class="message-body">
                <p>{% trans "To reset your password please enable JavaScript." context "forgotten password form page" %}</p>
              </div>

            </div>
          </div>
        </noscript>

      </div>
    </div>

  </div>
</div>
{% endblock content %}