{% load i18n %}
<footer class="misago-footer">
  <div class="container">
    <div class="footer-content">

      <noscript>
        <p class="noscript-message">
          <i class="material-icon hidden-xs hidden-sm">info_outline</i>
          {% trans "For complete experience please enable JavaScript." context "forum footer" %}
        </p>
      </noscript>

      {% if TERMS_OF_SERVICE_URL or PRIVACY_POLICY_URL or settings.forum_footnote or footer_menu %}
        <ul class="list-inline footer-nav">
          {% if settings.forum_footnote %}
            <li class="site-footnote">
              {{ settings.forum_footnote }}
            </li>
          {% endif %}
          {% for link in footer_menu %}
            <li>
              <a href="{{ link.url }}"{% if link.css_class %} class="{{ link.css_class }}"{% endif %}{% if link.rel %} rel="{{ link.rel }}"{% endif %}{% if link.target_blank %} target="_blank"{% endif %}>
                {{ link.title }}
              </a>
            </li>
          {% endfor %}
          {% if TERMS_OF_SERVICE_URL %}
            <li>
              <a href="{{ TERMS_OF_SERVICE_URL }}">{% trans "Terms of service" context "forum footer" %}</a>
            </li>
          {% endif %}
          {% if PRIVACY_POLICY_URL %}
            <li>
              <a href="{{ PRIVACY_POLICY_URL }}">{% trans "Privacy policy" context "forum footer" %}</a>
            </li>
          {% endif %}
        </ul>
      {% endif %}

      <a href="http://misago-project.org" class="misago-branding">
        powered by <strong>misago</strong>
      </a>

    </div>
  </div>
</footer>
