from .urls import *  # pylint: disable=wildcard-import, unused-wildcard-import

handler403 = "misago.core.testproject.views.mock_custom_403_error_page"
handler404 = "misago.core.testproject.views.mock_custom_404_error_page"
