# Generated by Django 1.11.1 on 2017-06-03 22:15
import django.contrib.postgres.fields
from django.contrib.postgres.operations import HStoreExtension
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("misago_users", "0009_redo_partial_indexes")]

    operations = [
        HStoreExtension(),
        migrations.AddField(
            model_name="user",
            name="profile_fields",
            field=django.contrib.postgres.fields.HStoreField(default=dict),
        ),
    ]
