# Generated by Django 3.2.15 on 2023-05-13 20:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("misago_notifications", "0002_migrate_threads_subscriptions"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(
                fields=["-id", "user"], name="misago_noti_id_5a01fa_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(
                fields=["-id", "user", "is_read"], name="misago_noti_id_7b4a20_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(
                condition=models.Q(("is_read", False)),
                fields=["user", "is_read"],
                name="misago_noti_user_unread",
            ),
        ),
        migrations.AddIndex(
            model_name="notification",
            index=models.Index(
                fields=["user", "post", "is_read"],
                name="misago_noti_user_id_9b98fb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="watchedthread",
            index=models.Index(
                fields=["user", "-thread"], name="misago_noti_user_id_9cde34_idx"
            ),
        ),
    ]
