{% extends "misago/admin/conf/form.html" %}
{% load i18n misago_admin_form %}


{% block form-body %}
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Basic settings" context "admin captcha settings form" %}</legend>

    {% form_row form.captcha_type %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "reCAPTCHA" context "admin captcha settings form" %}</legend>

    {% form_row form.recaptcha_site_key %}
    {% form_row form.recaptcha_secret_key %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Question and answer" context "admin captcha settings form" %}</legend>

    {% form_row form.qa_question %}
    {% form_row form.qa_help_text %}
    {% form_row form.qa_answers %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Stop Forum Spam" context "admin captcha settings form" %}</legend>

    {% form_row form.enable_stop_forum_spam %}
    {% form_row form.stop_forum_spam_confidence %}

  </fieldset>
</div>
{% endblock form-body %}