from .analytics import AnalyticsSettingsForm
from .base import SettingsForm
from .captcha import CaptchaSettingsForm
from .general import GeneralSettingsForm
from .notifications import NotificationsSettingsForm
from .oauth2 import OAuth2SettingsForm
from .threads import ThreadsSettingsForm
from .users import UsersSettingsForm
