{% load i18n %}
{% if post.is_valid %}
  <div class="post-body" itemprop="articleBody">
    <article class="misago-markup">
      {{ post.content|safe }}
    </article>
  </div>
{% else %}
  <div class="post-body post-body-invalid" itemprop="articleBody">
    <p class="lead">{% trans "This post's contents cannot be displayed." context "post body invalid" %}</p>
    <p class="text-muted">{% trans "This error is caused by invalid post content manipulation." context "post body invalid" %}</p>
  </div>
{% endif %}