{% load i18n %}
<div class="modal fade" id="uploadCss" tabindex="-1" role="dialog">
  <div class="modal-dialog" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h5 class="modal-title">{% trans "Upload CSS" context "admin theme css upload form" %}</h5>
        <button type="button" class="close" data-dismiss="modal" aria-label="{% trans 'Close' context 'admin theme css upload form' %}">
          <span aria-hidden="true">&times;</span>
        </button>
      </div>
      <form action="{% url 'misago:admin:themes:upload-css' pk=theme.pk %}" method="post" enctype="multipart/form-data">
        {% csrf_token %}
        <div class="modal-body">
          <div class="form-group">
            <label for="uploadCssInput">{% trans "Select CSS files to upload" context "admin theme css upload form" %}:</label>
            <input type="file" class="form-control" id="uploadCssInput" name="assets" multiple>
          </div>
        </div>
        <div class="modal-footer">
          <button type="button" class="btn btn-light" data-dismiss="modal">{% trans "Cancel" context "admin theme css upload form" %}</button>
          <button class="btn btn-primary">{% trans "Upload" context "admin theme css upload form" %}</button>
        </div>
      </form>
    </div><!-- /.modal-content -->
  </div><!-- /.modal-dialog -->
</div><!-- /.modal -->