{% extends "misago/base.html" %}
{% load i18n misago_absoluteurl %}


{% block title %}{{ title }} | {{ block.super }}{% endblock %}


{% block og-title %}{{ title }}{% endblock %}
{% block twitter-title %}{{ title }}{% endblock %}


{% block og-url %}{% absoluteurl request_path %}{% endblock %}
{% block twitter-url %}{% absoluteurl request_path %}{% endblock %}


{% block content %}
<div class="page page-legal legal-privacy-policy">
  <div class="container page-header-container">
    <div class="page-header page-header-privacy-policy">
      <div class="page-header-bg-image">
        <div class="page-header-bg-overlay">
          <div class="page-header-image"></div>
          <div class="page-header-banner page-header-banner-privacy-policy">
            <div class="page-header-banner-bg-image">
              <div class="page-header-banner-bg-overlay">
                <h1>{{ title }}</h1>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
  <div class="legal-body container page-container">
    <article class="misago-markup">
      {{ text|safe }}
    </article>
  </div>
</div>
{% endblock content %}
