{% extends "misago/profile/base.html" %}
{% load i18n %}


{% block title %}{% trans "Posts" context "user profile page posts" %} | {{ block.super }}{% endblock %}


{% block og-title %}{% trans "Posts" context "user profile page posts" %} | {{ block.super }}{% endblock %}
{% block twitter-title %}{% trans "Posts" context "user profile page posts" %} | {{ block.super }}{% endblock %}


{% block section %}
<div class="profile-posts">

  <nav class="toolbar">
    <div class="toolbar-section toolbar-section-auto">
      <div class="toolbar-item">
        <h3>
          {% if is_authenticated_user %}
            {% blocktrans trimmed count posts=profile.posts context "user profile page posts" %}
              You have posted {{ posts }} message.
            {% plural %}
              You have posted {{ posts }} messages.
            {% endblocktrans %}
          {% else %}
            {% blocktrans trimmed count posts=profile.posts with username=profile context "user profile page posts" %}
              {{ username }} has posted {{ posts }} message.
            {% plural %}
              {{ username }} has posted {{ posts }} messages.
            {% endblocktrans %}
          {% endif %}
        </h3>
      </div>
    </div>
  </nav>

  {% if posts %}
    {% include "misago/profile/feed.html" %}
  {% else %}
    <p class="lead">
      {% if is_authenticated_user %}
        {% trans "You have posted no messages" context "user profile page posts" %}
      {% else %}
        {% blocktrans trimmed with username=profile.username context "user profile page posts" %}
          {{ username }} posted no messages.
        {% endblocktrans %}
      {% endif %}
    </p>
  {% endif %}

</div>
{% endblock section %}