{% load i18n %}
<div class="message-panel">
  <div class="message-icon">
    <span class="material-icon">info_outline</span>
  </div>
  <div class="message-body">
    <p class="lead">
      {% trans "No categories exist or you don't have permission to see them." context "categories page" %}
    </p>
  </div>
</div>