{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Change sign-in credentials" context "credentials changed page" %} | {{ block.super }}{% endblock %}


{% block content %}
<div class="page page-message page-message-success page-options-done">
  <div class="container page-container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">check</span>
      </div>

      <div class="message-body">
        <p class="lead">{{ message }}</p>
        <p>{% trans "From now you will need to use new credentials when signing in." context "credentials changed page" %}</p>
      </div>

    </div>
  </div>
</div>
{% endblock content %}