{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Page not available" context "error 403 page" %} | {{ block.super }}{% endblock %}


{% block meta-description %}
  {% if message %}
    {{ message }}
  {% else %}
    {% trans "You don't have permission to access this page." context "error 403 page" %}
  {% endif %}
{% endblock meta-description %}


{% block og-title %}{% trans "Page not available" context "error 403 page" %}{% endblock %}
{% block twitter-title %}{% trans "Page not available" context "error 403 page" %}{% endblock %}


{% block og-description %}
  {% if message %}
    {{ message }}
  {% else %}
    {% trans "You don't have permission to access this page." context "error 403 page" %}
  {% endif %}
{% endblock %}


{% block twitter-description %}
  {% if message %}
    {{ message }}
  {% else %}
    {% trans "You don't have permission to access this page." context "error 403 page" %}
  {% endif %}
{% endblock %}


{% block content %}
<div class="page page-error page-error-403">
  <div class="container page-container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">remove_circle_outline</span>
      </div>

      <div class="message-body">
        <p class="lead">{% trans "This page is not available." context "error 403 page" %}</p>
        {% if message %}
          <p>{{ message|force_escape|urlize|linebreaksbr }}</p>
        {% else %}
          <p>{% trans "You don't have permission to access this page." context "error 403 page" %}</p>
        {% endif %}
      </div>

    </div>
  </div>
</div>
{% endblock content %}
