# Generated by Django 4.2.7 on 2023-11-13 19:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("misago_categories", "0009_auto_20221101_2111"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rolecategoryacl",
            name="category_role",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="misago_categories.categoryrole",
            ),
        ),
    ]
