{% extends "misago/admin/generic/base.html" %}
{% load i18n %}


{% block title %}
{{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block view %}
<div class="card card-admin-form">
  <form role="form" method="post" enctype="multipart/form-data">
    {% csrf_token %}

    <h5 class="card-header">
      {% trans "Change settings" context "admin settings form" %}
    </h5>
    <div class="card-body">

      {% if form.is_bound and not form.is_valid %}
        {% for error in form.non_field_errors %}
          <div class="alert alert-danger" role="alert">
            {{ error }}
          </div>
        {% empty %}
          <div class="alert alert-danger" role="alert">
            {% trans "Form was completed with errors." context "admin settings form" %}
          </div>
        {% endfor %}
      {% endif %}

      {% block form-body %}{% endblock form-body %}

    </div>
    <div class="card-footer text-right">
      <button class="btn btn-primary btn-sm">
        {% trans "Save changes" context "admin settings form" %}
      </button>
    </div>

  </form>
</div><!-- /.form-panel -->
{% block after-form %}{% endblock after-form %}
{% endblock view %}