{% extends "misago/emails/base.html" %}
{% load i18n misago_absoluteurl %}


{% block content %}
{% blocktrans trimmed with user=user context "change forgotten password email" %}
{{ user }}, you are receiving this message because you want to change forgotten password to your account.
{% endblocktrans %}
<br>
<br>
{% blocktrans trimmed context "change forgotten password email" %}
To set new password to your account click the link below:
{% endblocktrans %}
<br>
<br>
<a href="{% absoluteurl 'misago:forgotten-password-change-form' pk=user.pk token=confirmation_token %}">{% trans "Set new password" context "change forgotten password email cta" %}</a>
<br>
{% endblock content %}
