# Generated by Django 2.2.3 on 2019-09-15 19:15

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="MenuItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("url", models.URLField()),
                (
                    "menu",
                    models.CharField(
                        choices=[
                            ("both", "Navbar and footer"),
                            ("navbar", "Navbar"),
                            ("footer", "Footer"),
                        ],
                        max_length=6,
                    ),
                ),
                ("order", models.IntegerField(default=0)),
                ("css_class", models.CharField(blank=True, max_length=255, null=True)),
                ("target_blank", models.BooleanField(default=False)),
                ("rel", models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={"ordering": ("order",), "get_latest_by": "order"},
        )
    ]
