{% load i18n %}
{% with ban.get_serialized_message.message as ban_message %}
  {% if ban_message.html %}
    <div class="lead">
      {{ ban_message.html|safe }}
    </div>
  {% else %}
    <p class="lead">
      {{ ban_message.plain }}
    </p>
  {% endif %}
{% endwith %}
<p class="message-footnote">
  {% if ban.expires_on %}
    {% blocktrans trimmed with expires_on=ban.expires_on|date:"DATETIME_FORMAT" %}
      This ban expires on {{ expires_on }}.
    {% endblocktrans %}
  {% else %}
    {% trans "This ban is permanent." context "error banned message" %}
  {% endif %}
</p>