{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}
{{ settings.index_title|default:settings.forum_name }}
{% endblock title %}


{% block meta-description %}{{ settings.index_meta_description:default:"" }}{% endblock meta-description %}


{% block content %}
<div class="container">
  <p class="lead">This is Django-rendered forum index template.</p>
  <p>Neat, huh?</p>
</div>
{% endblock content %}
