{% extends "misago/threadslist/base.html" %}
{% load i18n misago_shorthands misago_stringutils %}


{% block title %}
{% if list_name %}
  {{ list_name }} | {{ category }} | {{ block.super }}
{% else %}
  {{ category }} | {{ block.super }}
{% endif %}
{% endblock title %}


{% block extra-css %}
page-private-threads
{% endblock extra-css %}


{% block page-header %}
<div class="container page-header-container">
  <div class="page-header page-header-private-threads">
    <div class="page-header-bg-image">
      <div class="page-header-bg-overlay">
        <div class="page-header-image"></div>
        <div class="page-header-banner page-header-banner-private-threads">
          <div class="page-header-banner-bg-image">
            <div class="page-header-banner-bg-overlay">
              <h1>{{ category }}</h1>
            </div>
          </div>
        </div>
        <div class="page-header-details">
          <div class="page-header-message">
            <p>{% trans "Private threads are threads which only those that started them and those they have invited may see and participate in." context "private threads page" %}</p>
          </div>
        </div>
      </div>
    </div>
  </div>
</div>
{% endblock page-header %}


{% block list-container %}
{% include "misago/threadslist/toolbar.html" with hide_categories=True %}

{{ block.super }}
{% endblock list-container%}


{% block thread %}
{% include "misago/threadslist/thread.html" with hide_path=True %}
{% endblock thread %}


{% block list-empty-message %}
{% if list_type == 'all' %}
  <p class="lead">
    {% trans "You aren't participating in any private threads." context "private threads page" %}
  </p>
{% else %}
  {% trans "No threads matching specified criteria were found." context "private threads page" %}
{% endif %}
{% endblock list-empty-message %}