{% extends "misago/admin/conf/form.html" %}
{% load i18n misago_admin_form %}


{% block form-body %}
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Posting" context "admin threads settings form" %}</legend>

    {% form_row form.thread_title_length_min %}
    {% form_row form.thread_title_length_max %}
    
    {% form_row form.post_length_min %}
    {% form_row form.post_length_max %}

    {% form_row form.post_attachments_limit %}
    {% form_row form.unused_attachments_lifetime %}

    {% form_row form.hourly_post_limit %}
    {% form_row form.daily_post_limit %}
    
  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Threads lists" context "admin threads settings form" %}</legend>
    
    {% form_row form.threads_per_page %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Thread pages" context "admin threads settings form" %}</legend>
    
    {% form_row form.posts_per_page %}
    {% form_row form.posts_per_page_orphans %}
    {% form_row form.events_per_page %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Read-tracker" context "admin threads settings form" %}</legend>
    
    {% form_row form.readtracker_cutoff %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Attachment error images" context "admin threads settings form" %}</legend>

    {% with form.attachment_403_image_delete as delete_field %}
      {% with form_settings.attachment_403_image as setting %}
        {% form_image_row form.attachment_403_image delete_field=delete_field size=setting.image_size dimensions=setting.image_dimensions %}
      {% endwith %}
    {% endwith %}
    {% with form.attachment_404_image_delete as delete_field %}
      {% with form_settings.attachment_404_image as setting %}
        {% form_image_row form.attachment_404_image delete_field=delete_field size=setting.image_size dimensions=setting.image_dimensions %}
      {% endwith %}
    {% endwith %}

  </fieldset>
</div>
{% endblock form-body %}