{% load i18n misago_capture %}
<div class="post-body post-body-hidden" itemprop="articleBody">
  <p class="lead">
    {% trans "This post is hidden. You cannot see its contents." context "post body hidden" %}
  </p>
  {% capture trimmed as hidden_by %}
    {% if post.hidden_by_id %}
      <a href="{% url 'misago:user' pk=post.hidden_by_id slug=post.hidden_by_slug %}" class="item-title">{{ post.hidden_by_name }}</a>
    {% else %}
      <span class="item-title">{{ post.hidden_by_name }}</span>
    {% endif %}
  {% endcapture %}
  <p class="text-muted">
    {% blocktrans trimmed with hidden_by=hidden_by|safe hidden_on=post.hidden_on|date:'DATETIME_FORMAT' context "post body hidden" %}
      Hidden by {{ hidden_by }} on {{ hidden_on }}.
    {% endblocktrans %}
  </p>
</div>