{% load i18n %}
<div class="row">
  <div class="col">
    <div class="card card-admin-info">
      <div class="card-body">
        <h4 class="card-title">
          {% trans "Users" context "admin dashboard totals" %}
        </h4>
      </div>
      <div class="card-body border-top card-admin-stat">
        {{ totals.users }}
      </div>
    </div>
  </div><!-- /.col -->
  <div class="col">
    <div class="card card-admin-info">
      <div class="card-body">
        <h4 class="card-title">
          {% trans "Threads" context "admin dashboard totals" %}
        </h4>
      </div>
      <div class="card-body border-top card-admin-stat">
        {{ totals.threads }}
      </div>
    </div>
  </div><!-- /.col -->
  <div class="col">
    <div class="card card-admin-info">
      <div class="card-body">
        <h4 class="card-title">
          {% trans "Posts" context "admin dashboard totals" %}
        </h4>
      </div>
      <div class="card-body border-top card-admin-stat">
        {{ totals.posts }}
      </div>
    </div>
  </div><!-- /.col -->
  <div class="col">
    <div class="card card-admin-info">
      <div class="card-body">
        <h4 class="card-title">
          {% trans "Attachments" context "admin dashboard totals" %}
        </h4>
      </div>
      <div class="card-body border-top card-admin-stat">
        {{ totals.attachments }}
      </div>
    </div>
  </div><!-- /.col -->
</div>