{% load i18n %}
{% if thread_notifications == "SITE_AND_EMAIL" %}
  <span class="material-icon">mail</span>
  {% trans "Watching" context "watch thread" %}
{% elif thread_notifications == "SITE_ONLY" %}
  <span class="material-icon">notifications_active</span>
  {% trans "Watching" context "watch thread" %}
{% else %}
  <span class="material-icon">notifications_none</span>
  {% trans "Watch" context "watch thread" %}
{% endif %}