{% extends "misago/admin/conf/form.html" %}
{% load i18n misago_admin_form %}


{% block form-body %}
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Default notifications settings" context "admin notifications settings form" %}</legend>
    <p>{% trans "Those settings are applied only to newly created user accounts." context "admin notifications settings form" %}</p>

    {% form_row form.watch_started_threads %}
    {% form_row form.watch_replied_threads %}
    {% form_row form.notify_new_private_threads_by_followed %}
    {% form_row form.notify_new_private_threads_by_other_users %}
    {% form_row form.watch_new_private_threads_by_followed %}
    {% form_row form.watch_new_private_threads_by_other_users %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Performance settings" context "admin notifications settings form" %}</legend>

    {% form_row form.delete_notifications_older_than %}

  </fieldset>
</div>
{% endblock form-body %}