# Generated by Django 2.2.1 on 2019-05-18 16:31

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("misago_users", "0018_auto_20190511_2051")]

    operations = [
        migrations.RunSQL("DROP INDEX IF EXISTS misago_user_is_staf_bf68aa_part"),
        migrations.RunSQL("DROP INDEX IF EXISTS misago_user_require_05204a_part"),
        migrations.RunSQL("DROP INDEX IF EXISTS misago_user_is_dele_2798b0_part"),
    ]
