# Generated by Django 4.2.7 on 2023-11-13 19:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("misago_users", "0024_user_notifications"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="rank",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="misago_users.rank",
            ),
        ),
    ]
