# Generated by Django 1.11.16 on 2018-11-25 15:15
from django.db import migrations, models

from .. import utils


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CacheVersion",
            fields=[
                (
                    "cache",
                    models.CharField(max_length=128, primary_key=True, serialize=False),
                ),
                (
                    "version",
                    models.CharField(
                        default=utils.generate_version_string, max_length=8
                    ),
                ),
            ],
        )
    ]
