{% extends "misago/admin/conf/form.html" %}
{% load i18n misago_admin_form %}


{% block form-body %}
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Site details" context "admin general settings form" %}</legend>

    {% form_row form.forum_name %}
    {% form_row form.forum_address %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Logo" context "admin general settings form" %}</legend>

    {% form_row form.logo_text %}
    {% with form.logo_delete as delete_field %}
      {% with form_settings.logo as setting %}
        {% form_image_row form.logo delete_field=delete_field size=setting.image_size dimensions=setting.image_dimensions %}
      {% endwith %}
    {% endwith %}
    {% with form.logo_small_delete as delete_field %}
      {% with form_settings.logo_small as setting %}
        {% form_image_row form.logo_small delete_field=delete_field size=setting.image_size dimensions=setting.image_dimensions %}
      {% endwith %}
    {% endwith %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Open Graph image" context "admin general settings form" %}</legend>

    {% with form.og_image_delete as delete_field %}
      {% with form_settings.og_image as setting %}
        {% form_image_row form.og_image delete_field=delete_field size=setting.image_size dimensions=setting.image_dimensions %}
      {% endwith %}
    {% endwith %}
    {% form_row form.og_image_avatar_on_profile %}
    {% form_row form.og_image_avatar_on_thread %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Forum index" context "admin general settings form" %}</legend>

    {% form_row form.index_title %}
    {% form_row form.index_meta_description %}
    {% form_row form.index_header %}
    {% form_row form.index_message %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Forum footnote" context "admin general settings form" %}</legend>

    {% form_row form.forum_footnote %}
    {% form_row form.email_footer %}

  </fieldset>
</div>
<div class="form-fieldset">
  <fieldset>
    <legend>{% trans "Admin Control Panel" context "admin general settings form" %}</legend>

    {% form_row form.show_admin_panel_link_in_ui %}

  </fieldset>
</div>
{% endblock form-body %}