{% extends "misago/base.html" %}
{% load i18n %}


{% block title %}{% trans "Change sign-in credentials" context "credentials changed page" %} | {{ block.super }}{% endblock %}


{% block content %}
<div class="page page-error page-options-error">
  <div class="container page-container">
    <div class="message-panel">

      <div class="message-icon">
        <span class="material-icon">info_outline</span>
      </div>

      <div class="message-body">
        <p class="lead">{% trans "Change confirmation link is invalid." context "credentials changed error page" %}</p>
        <p>{% trans "The confirmation link belongs to other user, was already used, or has expired." context "credentials changed error page" %}</p>
        <p>
          <a href="{% url 'misago:options-form' form_name='sign-in-credentials' %}">
            {% trans "Request another confirmation link." context "credentials changed page cta" %}
          </a>
        </p>
      </div>

    </div>
  </div>
</div>
{% endblock content %}