{% load i18n misago_admin_form %}
<div class="form-fieldset">
  <fieldset>
    <legend>{{ form.legend }}</legend>
    {% for field in form %}
      {% form_row field %}
    {% endfor %}
  </fieldset>
</div>