{% load i18n %}
{% if not THREADS_ON_INDEX and settings.index_header %}
  <div class="container page-header-container">
    <div class="page-header page-header-forum-index">
      <div class="page-header-bg-image">
        <div class="page-header-bg-overlay">
          <div class="page-header-image"></div>
          <div class="page-header-banner page-header-banner-forum-index">
            <div class="page-header-banner-bg-image">
              <div class="page-header-banner-bg-overlay">
                <h1>{{ settings.index_header }}</h1>
              </div>
            </div>
          </div>
          {% if settings.index_message %}
            <div class="page-header-details">
              <div class="page-header-message">
                {{ settings.index_message|force_escape|urlize|linebreaks|safe }}
              </div>
            </div>
          {% endif %}
        </div>
      </div>
    </div>
  </div>
{% elif THREADS_ON_INDEX %}
  <div class="container page-header-container">
    <div class="page-header page-header-categories">
      <div class="page-header-bg-image">
        <div class="page-header-bg-overlay">
          <div class="page-header-image"></div>
          <div class="page-header-banner page-header-banner-categories">
            <div class="page-header-banner-bg-image">
              <div class="page-header-banner-bg-overlay">
                <h1>{% trans "Categories" context "categories page" %}</h1>
              </div>
            </div>
          </div>
        </div>
      </div>
    </div>
  </div>
{% endif %}