from django.db import migrations


class Migration(migrations.Migration):
    """Migration superseded by 0016"""

    dependencies = [
        ("misago_users", "0002_users_settings"),
        ("misago_core", "0001_initial"),
    ]

    operations = []
