# Generated by Django 4.2.7 on 2023-12-05 19:22

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("misago_users", "0025_alter_user_rank"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="plugin_data",
            field=models.JSONField(default=dict),
        ),
        migrations.AddIndex(
            model_name="user",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["plugin_data"], name="misago_user_plugin__c45a32_gin"
            ),
        ),
    ]
